/***********************************************************************************************/
/***     Class: PwmOut, PwmOut.cpp                                                           ***/
/***********************************************************************************************/
#include <rxduino.h> 
#include "PwmOut.h"
#include "rxbed_iodefine.h"
#include "Assert.h"

// #define DEBUG 

#define DEF_FREQ  50         // mbed default frequency
#define MAX_DUTY 255         // GR duty 0 - 255
#define MIN_FREQ 12          // GR PWM Min Frequency
#define MAX_FREQ 184999      // GR PWM Max Frequency

#define PwmMAX 16            // MAX PWM instances

static int curr_freq ;      // current frequency
  
int PwmOut::countInstance = 0 ; 
 
PwmOut::PwmOut(PinName pin, const char *name){
        PwmOut::countInstance ++ ;
        MblibAssert("PwmOut", ((PwmOut::countInstance) <= PwmMAX), "Too many instances(%d).", PwmOut::countInstance) ;
        
        p = IOport[pin].gr_pin ;
        ObjName = name ;
        // pinMode(p, OUTPUT);
        analogWriteFrequency(DEF_FREQ) ;
        curr_freq = DEF_FREQ ;
        duty = 0.0F ;
    }
   
    void PwmOut::write(float value) {
    // Set the ouput duty-cycle, specified as a percentage (float)
    // Variables
    // value    A floating-point value representing the output duty-cycle, specified as a percentage.  The value should lie between 0.0f (representing on 0%) and 1.0f (representing on 100%).  Values outside this range will be saturated to 0.0f or 1.0f.
        value = MblibSatulation(value, 0.0, 1.0) ;
        int intv  = (int) (MAX_DUTY * (duty = value)) ; 
        intv = intv<0 ? 0 : intv ;
        intv = intv > MAX_DUTY ? MAX_DUTY : intv ;
        // digitalWrite(p, 0) ;
        analogWrite(p, intv);
#ifdef DEBUG
        printf("Pwm.write=%d, %d, %f\n", p, intv, value) ;
#endif
}
    
    float PwmOut::read() {
    // Return the current output duty-cycle setting, measured as a percentage (float)
    // Variables
    // returns    A floating-point value representing the current duty-cycle being output on the pin, measured as a percentage.  The returned value will lie between 0.0f (representing on 0%) and 1.0f (representing on 100%).
    // Note
    // This value may not match exactly the value set by a previous write.
        
        return(duty) ;
    }

    void PwmOut::period(float seconds) {
    // Set the PWM period, specified in seconds (float), keeping the duty cycle the same.
    // Note
    // The resolution is currently in microseconds; periods smaller than this will be set to zero.
    int freq ;
        MblibAssertFloatPositive("PwmOut.period", seconds) ;
        freq = 1.0 / seconds ;
		MblibAssertIntmin_max("PwmOut.period", freq, MIN_FREQ, MAX_FREQ) ;       
        analogWriteFrequency(freq) ;
        curr_freq = freq ;  // propagate it to all the PWMs
    }

void PwmOut::period_ms(int ms) 
// Set the PWM period, specified in milli-seconds (int), keeping the duty cycle the same.
    {
    int freq ;
        MblibAssertIntPositive("PwmOut.period_ms", ms) ;
        freq = 1000 / ms ;
		MblibAssertIntmin_max("PwmOut.period_ms", freq, MIN_FREQ, MAX_FREQ) ;    
        analogWriteFrequency(freq) ;
        curr_freq = freq ;  // propagate it to all the PWMs
    }

    void PwmOut::period_us(int us) {
    // Set the PWM period, specified in micro-seconds (int), keeping the duty cycle the same.
    int freq ;
        MblibAssertIntPositive("PwmOut.period_us", us) ;
        freq = 1000000 / us ;
		MblibAssertIntmin_max("PwmOut.period_us", freq, MIN_FREQ, MAX_FREQ) ;         
        analogWriteFrequency(freq) ;
        curr_freq = freq ;  // propagate it to all the PWMs
    }

    void PwmOut::pulsewidth(float seconds) {
    // Set the PWM pulsewidth, specified in seconds (float), keeping the period the same.
        MblibAssertFloatPositive("PwmOut.pulsewidth", seconds) ;
        duty = seconds * curr_freq ;
        duty = MblibSatulation(duty, 0.0, 1.0) ;
        analogWrite(p, (int) (MAX_DUTY * duty)); 
    }
    
    void PwmOut::pulsewidth_ms(int ms) {
    // Set the PWM pulsewidth, specified in milli-seconds (int), keeping the period the same.
        MblibAssertIntPositive("PwmOut.pulsewidth", ms) ;
        duty = ((float)ms * curr_freq) / 1000.0;
        duty = MblibSatulation(duty, 0.0, 1.0) ;
        analogWrite(p, (int) (MAX_DUTY * duty)); 
    }
        
    void PwmOut::pulsewidth_us(int us) {
    // Set the PWM pulsewidth, specified in micro-seconds (int), keeping the period the same.
        MblibAssertIntPositive("PwmOut.pulsewidth", us) ;
        duty = ((float)us * curr_freq) / 1000000.0;
        duty = MblibSatulation(duty, 0.0, 1.0) ;
        analogWrite(p, (int) (MAX_DUTY * duty)); 
    }
    #ifdef DEBUG
    static int lastv ;
    #endif
    PwmOut& PwmOut::operator= (float value) {
    // A operator shorthand for write()
        value = MblibSatulation(value, 0.0, 1.0) ;
        int intv  = (int) (MAX_DUTY * (duty = value)) ; 
        intv = intv<0 ? 0 : intv ;
        intv = intv > MAX_DUTY ? MAX_DUTY : intv ;
        // digitalWrite(p, 0) ;
        analogWrite(p, intv);
#ifdef DEBUG
        if(intv != lastv)
            printf("Pwm.write=%d, %f\n", intv, value) ;
        lastv = intv ;
#endif
}


PwmOut::operator float() {
   // An operator shorthand for read()
    return(duty) ;
}
